@@ -242,16 +242,16 @@ Soprano::Node Soprano::ODBC::QueryResult::getData( int colNum )
                 setError( Virtuoso::convertSqlError( SQL_HANDLE_STMT, d->m_hstmt, QLatin1String( "SQLGetDescField SQL_DESC_COL_DT_DT_TYPE failed" ) ) );
                 return Node();
             }
-            QUrl type;
+            QVariant::Type type;
             switch( dv_dt_type ) {
             case VIRTUOSO_DT_TYPE_DATE:
-                type = Vocabulary::XMLSchema::date();
+                type = QVariant::Date;
                 break;
             case VIRTUOSO_DT_TYPE_TIME:
-                type = Vocabulary::XMLSchema::time();
+                type = QVariant::Time;
                 break;
             default:
-                type = Vocabulary::XMLSchema::dateTime();
+                type = QVariant::DateTime;
                 break;
             }
             QString dts = QString::fromUtf8( reinterpret_cast<const char*>( data ) );
@@ -265,11 +265,12 @@ Soprano::Node Soprano::ODBC::QueryResult::getData( int colNum )
             //
             // node is an IRI ID
             //
-            // This type is only returned in output:valmode "LONG"
-            // It needs to be translated into a literal string using the
+            // It needs to be translated into a URIusing the
             // ID_TO_IRI() function as the value is database specific.
             //
-            setError( QLatin1String( "IRI_ID is not supported yet." ) );
+            // For now, we simply pass it on as a string literal
+            //
+            node = LiteralValue(QString::fromLatin1(reinterpret_cast<const char*>(data), length));
             break;
 
         case 204:
