@@ -22,6 +22,7 @@
 #include "virtuosobackend.h"
 #include "virtuosocontroller.h"
 #include "virtuosomodel.h"
+#include "virtuosomodel_p.h"
 #include "virtuosoconfigurator.h"
 #include "odbcconnection.h"
 #include "odbcconnectionpool.h"
@@ -32,6 +33,9 @@
 #include <QtCore/QtPlugin>
 #include <QtCore/QDir>
 
+#ifdef Q_OS_WIN
+#include <QtCore/QCoreApplication>
+#endif
 
 Q_EXPORT_PLUGIN2(soprano_virtuosobackend, Soprano::Virtuoso::BackendPlugin)
 
@@ -76,6 +80,9 @@ Soprano::StorageModel* Soprano::Virtuoso::BackendPlugin::createModel( const Back
     QString path = valueInSettings( settings, BackendOptionStorageDir ).toString();
     bool debugMode = valueInSettings( settings, BackendOptionUser, QLatin1String( "debugmode" ) ).toBool();
     int queryTimeout = valueInSettings( settings, QLatin1String( "QueryTimeout" ), 0 ).toInt();
+    bool disableStatementSignals = valueInSettings( settings, QLatin1String( "noStatementSignals" ), false ).toBool();
+    bool enableFakeBookleans = valueInSettings( settings, QLatin1String( "fakeBooleans" ), true ).toBool();
+    bool enableEmptyGraphs = valueInSettings( settings, QLatin1String( "emptyGraphs" ), true ).toBool();
 
     VirtuosoController* controller = 0;
     QString virtuosoVersion = QLatin1String("1.0.0"); // a default low version in case we connect to a running server
@@ -145,7 +152,9 @@ Soprano::StorageModel* Soprano::Virtuoso::BackendPlugin::createModel( const Back
         return 0;
     }
 
-    VirtuosoModel* model = new VirtuosoModel( virtuosoVersion, connectionPool, this );
+    VirtuosoModel* model = new VirtuosoModel( virtuosoVersion, connectionPool,
+                                              enableFakeBookleans, enableEmptyGraphs, this );
+    model->d->m_noStatementSignals = disableStatementSignals;
     // mem mangement the ugly way
     // FIXME: improve
     if ( controller ) {
