@@ -1,7 +1,7 @@
 /*
  * This file is part of Soprano Project
  *
- * Copyright (C) 2008-2011 Sebastian Trueg <trueg@kde.org>
+ * Copyright (C) 2008-2012 Sebastian Trueg <trueg@kde.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -47,56 +47,6 @@ namespace {
         }
     }
 
-    QString statementToConstructGraphPattern( const Soprano::Statement& s, bool withContext = false )
-    {
-        QString query;
-
-        if ( withContext ) {
-            query += QLatin1String( "graph " );
-            if ( s.context().isValid() ) {
-                query += nodeToN3( s.context() );
-            }
-            else {
-                query += QLatin1String( "?g" );
-            }
-            query += QLatin1String( " { " );
-        }
-
-        if ( s.subject().isValid() ) {
-            query += nodeToN3( s.subject() ) + ' ';
-        }
-        else {
-            query += QLatin1String( "?s " );
-        }
-
-        if ( s.predicate().isValid() ) {
-            query += nodeToN3( s.predicate() ) + ' ';
-        }
-        else {
-            query += QLatin1String( "?p " );
-        }
-
-        if ( s.object().isValid() ) {
-            if ( s.object().literal().isBool() )
-                query += Soprano::Node( Soprano::LiteralValue::fromString( s.object().literal().toBool() ? QString( QLatin1String( "true" ) ) : QLatin1String("false"),
-                                                                           Soprano::Virtuoso::fakeBooleanType() ) ).toN3();
-            else if ( s.object().literal().isByteArray() )
-                query += Soprano::Node( Soprano::LiteralValue::fromString( s.object().literal().toString(),
-                                                                           Soprano::Virtuoso::fakeBase64BinaryType() ) ).toN3();
-            else
-                query += nodeToN3( s.object() );
-        }
-        else {
-            query += QLatin1String( "?o" );
-        }
-
-        if ( withContext ) {
-            query += QLatin1String( " . }" );
-        }
-
-        return query;
-    }
-
     // there is still a bug in Virtuoso which makes the define prefix unusable: if a query defines the
     // graph the result will be empty if the exclude graph is specified.
     const char* s_queryPrefix =
@@ -105,23 +55,69 @@ namespace {
 //         "define input:named-graph-exclude <http://www.openlinksw.com/schemas/virtrdf#>";
 }
 
-
-Soprano::QueryResultIterator Soprano::VirtuosoModelPrivate::sparqlQuery( const QString& query )
+QString Soprano::VirtuosoModelPrivate::statementToConstructGraphPattern( const Soprano::Statement& s,
+                                                                         bool withContext ) const
 {
-    QString finalQuery( query );
-    finalQuery.prepend( QLatin1String( s_queryPrefix )  );
+    QString query;
 
-//    qDebug() << Q_FUNC_INFO << finalQuery;
+    if ( withContext ) {
+        query += QLatin1String( "graph " );
+        if ( s.context().isValid() ) {
+            query += nodeToN3( s.context() );
+        }
+        else {
+            query += QLatin1String( "?g" );
+        }
+        query += QLatin1String( " { " );
+    }
 
+    if ( s.subject().isValid() ) {
+        query += nodeToN3( s.subject() ) + ' ';
+    }
+    else {
+        query += QLatin1String( "?s " );
+    }
+
+    if ( s.predicate().isValid() ) {
+        query += nodeToN3( s.predicate() ) + ' ';
+    }
+    else {
+        query += QLatin1String( "?p " );
+    }
+
+    if ( s.object().isValid() ) {
+        if ( m_fakeBooleans && s.object().literal().isBool() )
+            query += Soprano::Node( Soprano::LiteralValue::fromString( s.object().literal().toBool() ? QString( QLatin1String( "true" ) ) : QLatin1String("false"),
+                                                                        Soprano::Virtuoso::fakeBooleanType() ) ).toN3();
+        else if ( s.object().literal().isByteArray() )
+            query += Soprano::Node( Soprano::LiteralValue::fromString( s.object().literal().toString(),
+                                                                        Soprano::Virtuoso::fakeBase64BinaryType() ) ).toN3();
+        else
+            query += nodeToN3( s.object() );
+    }
+    else {
+        query += QLatin1String( "?o" );
+    }
+
+    if ( withContext ) {
+        query += QLatin1String( " . }" );
+    }
+
+    return query;
+}
+
+
+Soprano::QueryResultIterator Soprano::VirtuosoModelPrivate::sqlQuery( const QString& query )
+{
     if ( ODBC::Connection* conn = connectionPool->connection() ) {
-        ODBC::QueryResult* result = conn->executeQuery( finalQuery );
+        ODBC::QueryResult* result = conn->executeQuery( query );
         if ( result ) {
             q->clearError();
             Virtuoso::QueryResultIteratorBackend* backend = new Virtuoso::QueryResultIteratorBackend( this, result );
             return backend;
         }
         else {
-            qDebug() << "Query failed:" << finalQuery;
+            qDebug() << "Query failed:" << query;
             q->setError( conn->lastError() );
             return 0;
         }
@@ -132,21 +128,32 @@ Soprano::QueryResultIterator Soprano::VirtuosoModelPrivate::sparqlQuery( const Q
     }
 }
 
+Soprano::QueryResultIterator Soprano::VirtuosoModelPrivate::sparqlQuery( const QString& query )
+{
+    return sqlQuery( QLatin1String( s_queryPrefix ) + query );
+}
+
 
 QString Soprano::VirtuosoModelPrivate::replaceFakeTypesInQuery( const QString& query )
 {
+    if( !m_fakeBooleans )
+        return query;
+
     QMutexLocker lock( &m_fakeBooleanRegExpMutex );
     return QString(query).replace( m_fakeBooleanRegExp, QString::fromLatin1("'\\2'^^<%1>").arg( Virtuoso::fakeBooleanTypeString() ) );
 }
 
 
-Soprano::VirtuosoModel::VirtuosoModel( const QString &virtuosoVersion, ODBC::ConnectionPool* connectionPool, const Backend* b )
+Soprano::VirtuosoModel::VirtuosoModel( const QString &virtuosoVersion, ODBC::ConnectionPool* connectionPool,
+                                       bool supportFakeBooleans, bool emptyGraphs, const Backend* b )
     : StorageModel(b),
       d( new VirtuosoModelPrivate() )
 {
     d->q = this;
     d->m_virtuosoVersion = virtuosoVersion;
     d->connectionPool = connectionPool;
+    d->m_fakeBooleans = supportFakeBooleans;
+    d->m_supportEmptyGraphs = emptyGraphs;
 }
 
 
@@ -179,20 +186,29 @@ Soprano::Error::ErrorCode Soprano::VirtuosoModel::addStatement( const Statement&
     }
 
     Statement s( statement );
-    if ( !s.context().isValid() ) {
-        s.setContext( Virtuoso::defaultGraph() );
+    if( !s.context().isValid() ) {
+        if ( d->m_supportEmptyGraphs ) {
+            s.setContext( Virtuoso::defaultGraph() );
+        }
+        else {
+            qDebug() << Q_FUNC_INFO << "Cannot add invalid statement:" << statement;
+            setError( "Cannot add statement with invalid context", Error::ErrorInvalidArgument );
+            return Error::ErrorInvalidArgument;
+        }
     }
 
     QString insert = QString::fromLatin1("sparql insert into %1")
-                     .arg( statementToConstructGraphPattern( s, true ) );
+                     .arg( d->statementToConstructGraphPattern( s, true ) );
 
     if ( ODBC::Connection* conn = d->connectionPool->connection() ) {
         if ( conn->executeCommand( insert ) == Error::ErrorNone ) {
             clearError();
 
-            // FIXME: can this be done with SQL/RDF views?
-            emit statementAdded( statement );
-            emit statementsAdded();
+            if(!d->m_noStatementSignals) {
+                // FIXME: can this be done with SQL/RDF views?
+                emit statementAdded( statement );
+                emit statementsAdded();
+            }
 
             return Error::ErrorNone;
         }
@@ -232,8 +248,16 @@ bool Soprano::VirtuosoModel::containsStatement( const Statement& statement ) con
     }
 
     Statement s( statement );
-    if ( !statement.context().isValid() )
-        s.setContext( Virtuoso::defaultGraph() );
+    if( !statement.context().isValid() ) {
+        if ( d->m_supportEmptyGraphs ) {
+            s.setContext( Virtuoso::defaultGraph() );
+        } else {
+            qDebug() << Q_FUNC_INFO << "Invalid Context " << statement;
+            setError( "Found invalid context", Error::ErrorInvalidArgument );
+            return Error::ErrorInvalidArgument;
+        }
+    }
+
     return containsAnyStatement( s );
 }
 
@@ -244,9 +268,9 @@ bool Soprano::VirtuosoModel::containsAnyStatement( const Statement& statement )
 
     QString query;
     if ( statement.context().isValid() )
-        query = QString::fromLatin1( "ask from %1 where { %2 . }" ).arg( statement.context().toN3(), statementToConstructGraphPattern( statement, false ) );
+        query = QString::fromLatin1( "ask from %1 where { %2 . }" ).arg( statement.context().toN3(), d->statementToConstructGraphPattern( statement, false ) );
     else
-        query = QString::fromLatin1( "ask where { %1 . }" ).arg( statementToConstructGraphPattern( statement, true ) );
+        query = QString::fromLatin1( "ask where { %1 . }" ).arg( d->statementToConstructGraphPattern( statement, true ) );
 //     if ( VirtuosoStatementHandler* sh = d->connection.execute( "sparql " + query ) ) {
 //         bool b = sh->fetchScroll();
 //         delete sh;
@@ -265,10 +289,10 @@ Soprano::StatementIterator Soprano::VirtuosoModel::listStatements( const Stateme
     QString query;
     if ( partial.context().isValid() )
         query = QString::fromLatin1( "select * from %1 where { %2 . }" )
-                .arg( partial.context().toN3(), statementToConstructGraphPattern( partial, false ) );
+                .arg( partial.context().toN3(), d->statementToConstructGraphPattern( partial, false ) );
     else
         query = QString::fromLatin1( "select * where { %1 . FILTER(?g != <%2>) . }" )
-                .arg( statementToConstructGraphPattern( partial, true ),
+                .arg( d->statementToConstructGraphPattern( partial, true ),
                       QLatin1String( Virtuoso::openlinkVirtualGraphString() ) );
 //    qDebug() << "List Statements Query" << query;
     return d->sparqlQuery( query )
@@ -290,8 +314,14 @@ Soprano::Error::ErrorCode Soprano::VirtuosoModel::removeStatement( const Stateme
     }
 
     Statement s( statement );
-    if ( !s.context().isValid() ) {
-        s.setContext( Virtuoso::defaultGraph() );
+    if( !s.context().isValid() ) {
+        if ( d->m_supportEmptyGraphs ) {
+            s.setContext( Virtuoso::defaultGraph() );
+        } else {
+            qDebug() << Q_FUNC_INFO << "Cannot remove invalid statement:" << statement;
+            setError( "Cannot remove statement with invalid context", Error::ErrorInvalidArgument );
+            return Error::ErrorInvalidArgument;
+        }
     }
     else if ( s.context().uri() == Virtuoso::openlinkVirtualGraph() ) {
         setError( "Cannot remove statements from the virtual openlink graph. Virtuoso would not like that.", Error::ErrorInvalidArgument );
@@ -299,13 +329,15 @@ Soprano::Error::ErrorCode Soprano::VirtuosoModel::removeStatement( const Stateme
     }
 
     QString query = QString::fromLatin1( "delete from %1" )
-                    .arg( statementToConstructGraphPattern( s, true ) );
+                    .arg( d->statementToConstructGraphPattern( s, true ) );
 //    qDebug() << "removeStatement query:" << query;
     if ( ODBC::Connection* conn = d->connectionPool->connection() ) {
         if ( conn->executeCommand( QLatin1String( "sparql " ) + query ) == Error::ErrorNone ) {
-            // FIXME: can this be done with SQL/RDF views?
-            emit statementRemoved( statement );
-            emit statementsRemoved();
+            if(!d->m_noStatementSignals) {
+                // FIXME: can this be done with SQL/RDF views?
+                emit statementRemoved( statement );
+                emit statementsRemoved();
+            }
         }
         setError( conn->lastError() );
     }
@@ -337,8 +369,8 @@ Soprano::Error::ErrorCode Soprano::VirtuosoModel::removeAllStatements( const Sta
         else {
             query = QString::fromLatin1( "delete from %1 { %2 } where { %3 }" )
                     .arg( statement.context().isValid() ? statement.context().toN3() : QString( "?g" ),
-                          statementToConstructGraphPattern( statement, false ),
-                          statementToConstructGraphPattern( statement, true ) );
+                          d->statementToConstructGraphPattern( statement, false ),
+                          d->statementToConstructGraphPattern( statement, true ) );
 
         }
     }
@@ -349,19 +381,26 @@ Soprano::Error::ErrorCode Soprano::VirtuosoModel::removeAllStatements( const Sta
         // For versions before we need to use the old hacky method which requires iterating all graph candidates.
         //
         if( d->m_virtuosoVersion >= QLatin1String("6.1.5") ) {
-            query = QString::fromLatin1("delete { %1 } where { %1 }").arg( statementToConstructGraphPattern(statement, true) );
+            query = QString::fromLatin1("delete { %1 } where { %1 }").arg( d->statementToConstructGraphPattern(statement, true) );
         }
         else {
             QList<Node> allContexts = d->sparqlQuery( QString::fromLatin1( "select distinct ?g where { %1 . FILTER(?g != <%2>) . }" )
-                                                      .arg( statementToConstructGraphPattern( statement, true ),
+                                                      .arg( d->statementToConstructGraphPattern( statement, true ),
                                                             QLatin1String( Virtuoso::openlinkVirtualGraphString() ) ) )
                                       .iterateBindings( 0 ).allNodes();
             foreach( const Node& node, allContexts ) {
                 Statement s( statement );
                 if ( node.isValid() )
                     s.setContext( node );
-                else
-                    s.setContext( Virtuoso::defaultGraph() );
+                else {
+                    if( d->m_supportEmptyGraphs ) {
+                        s.setContext( Virtuoso::defaultGraph() );
+                    } else {
+                        qDebug() << Q_FUNC_INFO << "Cannot remove invalid statement:" << statement;
+                        setError( "Cannot remove statement with invalid context", Error::ErrorInvalidArgument );
+                        return Error::ErrorInvalidArgument;
+                    }
+                }
                 Error::ErrorCode c = removeAllStatements( s );
                 if ( c != Error::ErrorNone )
                     return c;
@@ -372,12 +411,22 @@ Soprano::Error::ErrorCode Soprano::VirtuosoModel::removeAllStatements( const Sta
 
     if ( ODBC::Connection* conn = d->connectionPool->connection() ) {
         if ( conn->executeCommand( "sparql " + query ) == Error::ErrorNone ) {
-            // FIXME: can this be done with SQL/RDF views?
-            emit statementsRemoved();
-            Statement signalStatement( statement );
-            if( signalStatement.context() == Virtuoso::defaultGraph() )
-                signalStatement.setContext( Node() );
-            emit statementRemoved( signalStatement );
+            if(!d->m_noStatementSignals) {
+                // FIXME: can this be done with SQL/RDF views?
+                emit statementsRemoved();
+                Statement signalStatement( statement );
+                if( signalStatement.context() == Virtuoso::defaultGraph() ) {
+                    if( d->m_supportEmptyGraphs ) {
+                        signalStatement.setContext( Node() );
+                    } else {
+                        qDebug() << Q_FUNC_INFO << "Cannot remove invalid statement:" << statement;
+                        setError( "Cannot remove statement with invalid context", Error::ErrorInvalidArgument );
+                        return Error::ErrorInvalidArgument;
+                    }
+                }
+
+                emit statementRemoved( signalStatement );
+            }
         }
         setError( conn->lastError() );
     }
@@ -416,13 +465,20 @@ Soprano::QueryResultIterator Soprano::VirtuosoModel::executeQuery( const QString
                                                                    Query::QueryLanguage language,
                                                                    const QString& userQueryLanguage ) const
 {
-    if ( language != Soprano::Query::QueryLanguageSparql ) {
+    if ( language == Soprano::Query::QueryLanguageSparql ) {
+        return d->sparqlQuery( d->replaceFakeTypesInQuery( query ) );
+    }
+
+    else if( language == Soprano::Query::QueryLanguageUser &&
+             userQueryLanguage.toLower() == QLatin1String("sql") ) {
+        return d->sqlQuery( d->replaceFakeTypesInQuery( query ) );
+    }
+
+    else {
         setError( Error::Error( QString::fromLatin1( "Unsupported query language %1." )
                                 .arg( Query::queryLanguageToString( language, userQueryLanguage ) ) ) );
         return QueryResultIterator();
     }
-
-    return d->sparqlQuery( d->replaceFakeTypesInQuery( query ) );
 }
 
 
