@@ -31,6 +31,9 @@ namespace Soprano {
     }
 
     class VirtuosoModelPrivate;
+    namespace Virtuoso {
+        class BackendPlugin;
+    }
 
     class VirtuosoModel : public StorageModel
     {
@@ -40,7 +43,8 @@ namespace Soprano {
         /**
          * \param connection The base connection used for non-transaction requests
          */
-        VirtuosoModel( const QString& virtuosoVersion, ODBC::ConnectionPool* connectionPool, const Backend* );
+        VirtuosoModel( const QString& virtuosoVersion, ODBC::ConnectionPool* connectionPool,
+                       bool supportFakeBooleans, bool emptyGraphs, const Backend* );
         ~VirtuosoModel();
 
         Error::ErrorCode addStatement( const Statement &statement );
@@ -68,6 +72,8 @@ namespace Soprano {
         VirtuosoModelPrivate* const d;
 
         friend class VirtuosoModelPrivate;
+        friend class VirtuosoBackend;
+        friend class Virtuoso::BackendPlugin;
     };
 }
 
